import pandas as pd
import numpy as np
from collections import defaultdict
from prettytable import PrettyTable

def get_paper_score(qualis):
    if qualis is None or qualis == 'C':
        return 0
    return get_journal_score({'A1':88, 'A2':76, 'A3':63, 'A4':51, 'B1':38, 'B2':26, 'B3':13, 'B4':11}[qualis])

def get_journal_score(perc):
    if perc >= 87.5:
        return 100
    elif perc >= 75:
        return 87.5
    elif perc >= 62.5:
        return 75
    elif perc >= 50:
        return 62.5
    elif perc >= 37.5:
        return 50
    elif perc >= 25:
        return 37.5
    elif perc >= 12.5:
        return 25
    return 12.5

def get_score(df, f):
    return sorted(list(map(f, df)), reverse=True)

def bonus_score(df, df_entry, count_students=True):
    obsv = []
    n_docs = len(df[df.titulo == df_entry.titulo].values)
    has_student = len(df_entry.alunos) > 0 and count_students
    score = df_entry.score
    if n_docs > 2:
        score = 2*score/n_docs
        obsv.append("penalização >2 docentes")
    if has_student:
        score *= 1.3
        obsv.append("bônus 30% aluno")
    return score, " -- ".join(obsv)

df = pd.read_csv("datasets/papers.csv")
df_books = pd.read_csv("datasets/books.csv")
df_extra = pd.read_csv("datasets/extras.csv", index_col=0)

journals = ~df.is_conf & ~df.percentile.isna()
confs = df.is_conf & ~df.qualis.isna()
df.loc[journals, "score" ] = list(map(get_journal_score, df[journals].percentile))
df.loc[confs, "score"] = list(map(get_paper_score, df[confs].qualis))
df.fillna(value={"alunos":""}, inplace=True)

if len(df_books) != 0:
    df_books.loc[df_books.is_chapter==1, "score"] = 25
    df_books.loc[df_books.is_chapter==0, "score"] = 100
    df_books.fillna(value={"alunos":""}, inplace=True)

table = PrettyTable()
table.field_names = ["Docente","Nota sem limite","Nota com limite","Total"]

personalTable = {}

for docente in df.Docente.unique():
    doc_df = df[df.Docente == docente]
    personalTable[docente] = PrettyTable()
    personalTable[docente].field_names = ["Tipo", "Titulo", "Alunos", "Pontos computados", "Observação"]

    top_perc = doc_df.percentile >= 50
    is_conf = doc_df.is_conf
    top_qualis = doc_df.qualis.isin(['A1','A2','A3','A4'])
    has_qualis = ~doc_df.qualis.isna()
    has_students = doc_df.alunos.notnull()

    top_journals = ~is_conf & top_perc
    bot_journals = ~is_conf & ~top_perc & has_students & ~doc_df.percentile.isna()
    top_confs = is_conf & top_qualis
    bot_confs = is_conf & ~top_qualis & has_qualis & has_students
    
    max_conf = 4 * (doc_df[top_journals].values.shape[0] + doc_df[bot_journals].values.shape[0] + 1)
    tot_ilimitado = 0

    for _, journal in doc_df[top_journals].iterrows():
        score, obs = bonus_score(df, journal)
        personalTable[docente].add_row(["Journal >=50", journal.titulo, journal.alunos, score, obs])
        tot_ilimitado += score
    for _, paper in doc_df[top_confs].iterrows():
        if max_conf == 0:
            score, obs = 0, "máximo conf. atingido"
        else:
            score, obs = bonus_score(df, paper)
            max_conf -= 1
        personalTable[docente].add_row(["Paper A", paper.titulo, paper.alunos, score, obs])
        tot_ilimitado += score

    tot_limitado = 0
    for _, journal in doc_df[bot_journals].iterrows():
        if tot_limitado == 100:
            score, obs = 0, "pontuação máxima atingida"
        else:
            score, obs = bonus_score(df, journal, False)
            if tot_limitado + score > 100:
                score = 100 - tot_limitado
                obs = "pontuação máxima atingida"
            if len(journal.alunos) == 0:
                score = 0
                obs = "sem alunos"
        personalTable[docente].add_row(["Journal <50", journal.titulo, journal.alunos, score, obs])
        tot_limitado += score
    for _, paper in doc_df[bot_confs].iterrows():
        if max_conf == 0:
            score, obs = 0, "máximo conf. atingido"
        elif tot_limitado == 100:
            score, obs = 0, "pontuação máxima atingida"
        elif len(paper.alunos) == 0:
            score, obs = 0, "sem alunos"
        else:
            score, obs = bonus_score(df, paper, False)
            if tot_limitado + score > 100:
                score = 100 - tot_limitado
                obs = "pontuação máxima atingida"
            max_conf -= 1
        personalTable[docente].add_row(["Paper B", paper.titulo, paper.alunos, score, obs])
        tot_limitado += score

    for _, chapter in df_books[(df_books.Docente == docente) & (df_books.is_chapter == True)].iterrows():
        if tot_limitado == 100:
            score, obs = 0, "pontuação máxima atingida"
        else:
            score, obs = bonus_score(df_books, chapter, True)
            if tot_limitado + score > 100:
                score = 100 - tot_limitado
                obs = "pontuação máxima atingida"
        personalTable[docente].add_row(["Capitulo de Livro", chapter.titulo, "", score, obs])
        tot_limitado += score

    for _, book in df_books[(df_books.Docente == docente) & (df_books.is_chapter == False)].iterrows():
        score, obs = bonus_score(df_books, book, True)
        personalTable[docente].add_row(["Livro", book.titulo, "", score, obs])
        tot_ilimitado += score


    doc_extra = df_extra.loc[docente]
    n_pat_nac = doc_extra.Patentes_nac
    n_pat_int = doc_extra.Patentes_int
    n_soft = doc_extra.Softwares
    n_cnpq = doc_extra.CNPq
    n_coord = doc_extra.Coord_proj
    n_projs = doc_extra.proj

    if n_pat_nac > 0:
        score = n_pat_nac * 25
        obs = ""
        if tot_limitado + score > 100:
            score = max(0, 100 - tot_limitado)
            obs = "pontuação máxima atingida"
        tot_limitado += score
        personalTable[docente].add_row(["Patente Nacional", "-", "", score, obs])
    if n_pat_int > 0:
        tot_ilimitado += n_pat_int * 70
        personalTable[docente].add_row(["Patente Internacional", "-", "", n_pat_int*70, ""])
    if n_soft > 0:
        score = max(30, n_soft*10)
        obs = ""
        if tot_limitado + score > 100:
            score = max(0, 100 - tot_limitado)
            obs = "pontuação máxima atingida"
        tot_limitado += score
        personalTable[docente].add_row(["Software", "-", "", score, obs])
    if n_cnpq > 0:
        tot_ilimitado += 100
        personalTable[docente].add_row(["Bolsista Produtividade", "-", "", 100, ""])
    if n_coord > 0:
        score = max(150, 50*n_coord)
        tot_ilimitado += score
        personalTable[docente].add_row(["Coordenação de Projetos Financiados", "-", "", score, ""])
    if n_projs > 0:
        score = max(50, 10*n_projs)
        obs = ""
        if tot_limitado + score > 100:
            score = max(0, 100 - tot_limitado)
            obs = "pontuação máxima atingida"
        tot_limitado += score
        personalTable[docente].add_row(["Participação em Projetos Financiados", "-", "", score, obs])


    table.add_row([docente,tot_ilimitado, tot_limitado, tot_ilimitado + tot_limitado])
   
    print(docente)
    print(personalTable[docente])

print(table.get_string(sortby="Total"))
